# models.py

from sentiment_data import *


# Feature extraction base type. Takes an example and returns an indexed list of features.
class FeatureExtractor(object):
    def get_indexer(self):
        raise Exception("Don't call me, call my subclasses")

    # Extract features. Includes a flag add_to_indexer to control whether the indexer should be expanded.
    # At test time, any unseen features should be discarded, but at train time, we probably want to keep growing it.
    def extract_features(self, ex: SentimentExample, add_to_indexer: bool=False):
        raise Exception("Don't call me, call my subclasses")


# Extracts unigram bag-of-words features from a sentence. It's up to you to decide how you want to handle counts
class UnigramFeatureExtractor(FeatureExtractor):
    def __init__(self, indexer: Indexer):
        self.indexer = indexer


# Bigram feature extractor analogous to the unigram one.
class BigramFeatureExtractor(FeatureExtractor):
    def __init__(self, indexer: Indexer):
        raise Exception("Must be implemented")


# Better feature extractor...try whatever techniques you can think of!
class BetterFeatureExtractor(FeatureExtractor):
    def __init__(self, indexer: Indexer):
        raise Exception("Must be implemented")


# Sentiment classifier base type
class SentimentClassifier(object):
    # Makes a prediction for the given
    def predict(self, ex: SentimentExample):
        raise Exception("Don't call me, call my subclasses")


# Always predicts the positive class
class TrivialSentimentClassifier(SentimentClassifier):
    def predict(self, ex: SentimentExample):
        return 1


# Implement this class -- you should at least have init() and implement the predict method from the SentimentClassifier
# superclass
class NaiveBayesClassifier(SentimentClassifier):
    def __init__(self):
        raise Exception("Must be implemented")


# Implement this class -- you should at least have init() and implement the predict method from the SentimentClassifier
# superclass
class PerceptronClassifier(SentimentClassifier):
    def __init__(self):
        raise Exception("Must be implemented")


# Train a Naive Bayes model on the given training examples using the given FeatureExtractor
def train_nb(train_exs: List[SentimentExample], feat_extractor: FeatureExtractor) -> NaiveBayesClassifier:
    raise Exception("Must be implemented")


# Train a Perceptron model on the given training examples using the given FeatureExtractor
def train_perceptron(train_exs: List[SentimentExample], feat_extractor: FeatureExtractor) -> PerceptronClassifier:
    raise Exception("Must be implemented")


# Main entry point for your modifications. Trains and returns one of several models depending on the args
# passed in from the main method.
def train_model(args, train_exs):
    # Initialize feature extractor
    if args.feats == "UNIGRAM":
        # Add additional preprocessing code here
        feat_extractor = UnigramFeatureExtractor(Indexer())
    elif args.feats == "BIGRAM":
        # Add additional preprocessing code here
        feat_extractor = BigramFeatureExtractor(Indexer())
    elif args.feats == "BETTER":
        # Add additional preprocessing code here
        feat_extractor = BetterFeatureExtractor(Indexer())
    else:
        raise Exception("Pass in UNIGRAM, BIGRAM, or BETTER to run the appropriate system")

    # Train the model
    if args.model == "TRIVIAL":
        model = TrivialSentimentClassifier()
    elif args.model == "NB":
        model = train_nb(train_exs, feat_extractor)
    elif args.model == "PERCEPTRON":
        model = train_perceptron(train_exs, feat_extractor)
    else:
        raise Exception("Pass in TRIVIAL, NB, or PERCEPTRON to run the appropriate system")
    return model